﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Graphics;

namespace XNA4
{
    public class FpsCounter : DrawableGameComponent
    {
        Game game;
        SpriteBatch spriteBatch;

        private float updateInterval = 1.0f;
        private float timeSinceLastUpdate = 0.0f;
        private float framecount = 0;

        private float fps = 0;

        // draw text 
        InputHandler input;
        bool drawFPS = false;
        SpriteFont font;

         /// <summary>
        /// The frames per second.
        /// </summary>
        public float FPS
        {
            get { return fps; }
        }

        public FpsCounter(Game game)
            : base(game)
        {
            Enabled = true;
            this.game = game;
            input = (InputHandler)game.Services.GetService(typeof(InputHandler));
            input.KeyPressed += new EventHandler<KeyboardEventArgs>(PrintDebug);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            // load font
            font = this.game.Content.Load<SpriteFont>("Font/Font1");
        }

        /// <summary>
        /// Update the fps.
        /// </summary>
        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            
            if (input.IsKeyPressed(Keys.F))
                drawFPS = !drawFPS;

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;
            framecount++;
            timeSinceLastUpdate += elapsed;

            if (timeSinceLastUpdate > updateInterval)
            {
                fps = framecount / timeSinceLastUpdate; //mean fps over updateIntrval
                framecount = 0;
                timeSinceLastUpdate -= updateInterval;

                if (Updated != null)
                    Updated(this, new EventArgs());
            }
        }

        /// <summary>
        /// FpsCounter Updated Event.
        /// </summary>
        public event EventHandler<EventArgs> Updated;

        public void PrintDebug(object sender, KeyboardEventArgs e)
        {
            System.Diagnostics.Trace.WriteLine("PrintDebug " + e.Key);
        }

        public sealed override void Draw(GameTime gameTime)
        {
#if XBOX360
                System.Diagnostics.Debug.WriteLine("FPS: " + fps.ToString());
#else
                Game.Window.Title = "FPS: " + fps.ToString();
                if (drawFPS)
                {
                    // draw the text for fpsCounter
                    spriteBatch.Begin();
                    spriteBatch.DrawString(font, fps.ToString(), Vector2.One, Color.White);
                    spriteBatch.End();
                }
#endif
        }
    }
}
