﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace XNA4
{
    public class KeyboardEventArgs : EventArgs
    {
        public Keys Key;
    }

    class InputHandler : GameComponent
    {        
        public GamePadState CurrentGamePadState;
        public GamePadState PrevGamePadState;

        public KeyboardState CurrentKeyboardState;
        public KeyboardState PrevKeyboardState;
        
        public MouseState CurrentMouseState;
        public MouseState PrevMouseState; 

        private Point mouseLocation;  // mouse location
        public Point MouseLocation
        {
            get { return mouseLocation; }
        }

        private Vector2 mouseMoved;     // how much mouse moved
        public Vector2 MouseMoved
        {
            get { return mouseMoved; }
        }
        //public bool MouseLeftPressed;
        public bool MouseLeftClicked;

        /// <summary>
        /// InputHandler KeyPressed Event.
        /// </summary>
        public event EventHandler<KeyboardEventArgs> KeyPressed;

        public InputHandler(Game game) : base(game)
        {
            Enabled = true;
        }

        public override void Update(GameTime gameTime)
        {
            PrevGamePadState = CurrentGamePadState;
            CurrentGamePadState = GamePad.GetState(PlayerIndex.One);

            PrevKeyboardState = CurrentKeyboardState;
            CurrentKeyboardState = Keyboard.GetState();

            PrevMouseState = CurrentMouseState;
            CurrentMouseState = Mouse.GetState(); 

            mouseMoved = new Vector2(PrevMouseState.X - CurrentMouseState.X, PrevMouseState.Y - CurrentMouseState.Y);
            mouseLocation = new Point(CurrentMouseState.X, CurrentMouseState.Y);

            if ((CurrentMouseState.LeftButton == ButtonState.Pressed) && (PrevMouseState.LeftButton == ButtonState.Released))
            {
                MouseLeftClicked = true;
            }
            else
            {
                MouseLeftClicked = false;
            }
        }

        public Point MousePoint()
        {
            return mouseLocation;
        }

        public bool MouseLeftButtonPressed(ButtonState button)
        {
            return CurrentMouseState.LeftButton == button;
        }

        public bool MouseMiddleButtonPressed(ButtonState button)
        {
            return CurrentMouseState.MiddleButton == button;
        }

        public bool MouseRightButtonPressed(ButtonState button)
        {
            return CurrentMouseState.RightButton == button;
        }

        public bool IsKeyPressed(Keys key)
        {
            if (CurrentKeyboardState.IsKeyDown(key) && PrevKeyboardState.IsKeyUp(key))
            {
                // KeyPressed event
                if (KeyPressed != null)
                    KeyPressed(this, new KeyboardEventArgs() { Key = key });

                return true;
            }
            return false;
        }

        public bool IsKeyDown(Keys key)
        {
            return CurrentKeyboardState.IsKeyDown(key);
        }

        public bool PauseGame  
        {  
            get 
            {  
                return IsKeyPressed(Keys.Escape) ||
                    (CurrentGamePadState.Buttons.Back == ButtonState.Pressed &&  
                    PrevGamePadState.Buttons.Back == ButtonState.Released) ||
                    (CurrentGamePadState.Buttons.Start == ButtonState.Pressed &&  
                    PrevGamePadState.Buttons.Start == ButtonState.Released);  
            }  
        }
    }
}
